package geniusweb.protocol.tournament.allpermutations;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonTypeInfo;
import com.fasterxml.jackson.annotation.JsonValue;

import geniusweb.references.ProfileRef;

/**
 * Contains a list of profiles, one for each {@link Team} member.
 */
@JsonAutoDetect(fieldVisibility = Visibility.ANY, getterVisibility = Visibility.NONE, setterVisibility = Visibility.NONE, isGetterVisibility = Visibility.NONE)
@JsonTypeInfo(use = JsonTypeInfo.Id.NAME, include = JsonTypeInfo.As.WRAPPER_OBJECT)
public class ProfileList {
	@JsonValue
	private final List<ProfileRef> profiles; // field to ensure immutability

	/**
	 * 
	 * @param ps a list of {@link ProfileRef}s. The length and order is
	 *           important as each will be matched with the {@link Team} member
	 *           at the same index.
	 */
	@JsonCreator
	public ProfileList(List<ProfileRef> ps) {
		this.profiles = new ArrayList<>(ps);
	}

	public List<ProfileRef> getProfiles() {
		return Collections.unmodifiableList(profiles);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((profiles == null) ? 0 : profiles.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ProfileList other = (ProfileList) obj;
		if (profiles == null) {
			if (other.profiles != null)
				return false;
		} else if (!profiles.equals(other.profiles))
			return false;
		return true;
	}

	public String toString() {
		return profiles.toString();
	}
}
