package geniusweb.protocol.session.shaop;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;

import geniusweb.actions.Accept;
import geniusweb.actions.Action;
import geniusweb.actions.EndNegotiation;
import geniusweb.actions.Offer;
import geniusweb.actions.PartyId;
import geniusweb.issuevalue.Bid;
import geniusweb.progress.Progress;
import geniusweb.progress.ProgressRounds;
import geniusweb.progress.ProgressTime;
import geniusweb.protocol.ProtocolException;
import geniusweb.protocol.partyconnection.ProtocolToPartyConn;
import geniusweb.protocol.partyconnection.ProtocolToPartyConnections;
import geniusweb.protocol.session.SessionResult;
import geniusweb.protocol.session.TeamOfPartiesAndProfiles;
import tudelft.utilities.junit.GeneralTests;

public class SHAOPStateTest extends GeneralTests<SHAOPState> {
	private final long NOW = 1000;

	private PartyId party1 = new PartyId("party1");
	private PartyId party2 = new PartyId("party2");
	private PartyId party3 = new PartyId("party3");
	private PartyId party4 = new PartyId("party4");
	private List<Action> actions = new LinkedList<>();
	private Action action = mock(Action.class);
	private List<Action> actions2 = Arrays.asList(action);

	private ProtocolToPartyConn party1conn = mock(ProtocolToPartyConn.class);
	private ProtocolToPartyConn party2conn = mock(ProtocolToPartyConn.class);
	private ProtocolToPartyConn party3conn = mock(ProtocolToPartyConn.class);
	private ProtocolToPartyConn party4conn = mock(ProtocolToPartyConn.class);

	private ProtocolToPartyConnections connections = new ProtocolToPartyConnections(
			Arrays.asList(party1conn, party2conn));
	private ProtocolToPartyConnections connections2 = new ProtocolToPartyConnections(
			Arrays.asList(party1conn, party3conn));
	private ProtocolToPartyConnections connections3 = new ProtocolToPartyConnections(
			Arrays.asList(party1conn, party2conn, party3conn));
	private ProtocolToPartyConnections connections4 = new ProtocolToPartyConnections(
			Arrays.asList(party1conn, party2conn, party3conn, party4conn));

	private Progress progresstime = mock(ProgressTime.class);
	private ProgressRounds progressrounds = mock(ProgressRounds.class);
	private ProgressRounds progressrounds1 = mock(ProgressRounds.class);
	private SHAOPSettings settings = mock(SHAOPSettings.class);
	private SHAOPState state1, state1a, state2, state3, state4;

	private Bid bid1 = mock(Bid.class);
	private Bid otherbid = mock(Bid.class);
	private Accept accept1 = new Accept(party1, bid1);
	private Accept accept2 = new Accept(party2, bid1);
	private Accept acceptother = new Accept(party2, otherbid);
	private Offer offer1 = new Offer(party1, bid1);
	private EndNegotiation endNegotiation1 = new EndNegotiation(party1);
	private Map<PartyId, Integer> partynrs = new HashMap<>();

	private TeamOfPartiesAndProfiles team1 = mock(
			TeamOfPartiesAndProfiles.class);
	private TeamOfPartiesAndProfiles team2 = mock(
			TeamOfPartiesAndProfiles.class);

	@Before
	public void before() {
		when(party1conn.getParty()).thenReturn(party1);
		when(party2conn.getParty()).thenReturn(party2);
		when(party3conn.getParty()).thenReturn(party3);
		when(party1conn.toString()).thenReturn("conn1");
		when(party2conn.toString()).thenReturn("conn2");
		when(party3conn.toString()).thenReturn("conn3");
		when(party4conn.toString()).thenReturn("conn4");
		when(action.toString()).thenReturn("act1");
		when(settings.toString()).thenReturn("settings");
		when(progressrounds.toString()).thenReturn("progressrounds");
		when(progresstime.toString()).thenReturn("progresstime");

		when(progressrounds.getTerminationTime())
				.thenReturn(new Date(System.currentTimeMillis() + 3600000l));
		when(progressrounds1.getTerminationTime())
				.thenReturn(new Date(System.currentTimeMillis() + 3600000l));
		when(progressrounds.advance()).thenReturn(progressrounds1);

		partynrs.put(party1, 0);
		partynrs.put(party2, 1);
		partynrs.put(party3, 2);
		partynrs.put(party4, 3);
		Map<PartyId, Double> totalspent = new HashMap<>();

		state1 = new SHAOPState(actions, connections, progresstime, settings,
				null, 0, partynrs, totalspent);
		state1a = new SHAOPState(actions, connections, progresstime, settings,
				null, 0, partynrs, totalspent);
		state2 = new SHAOPState(actions2, connections, progresstime, settings,
				null, 0, partynrs, totalspent);
		state3 = new SHAOPState(actions, connections, progressrounds, settings,
				null, 0, partynrs, totalspent);
		state4 = new SHAOPState(actions, connections, progresstime, settings,
				null, 1, partynrs, totalspent);

		when(settings.getTeams()).thenReturn(Arrays.asList(team1, team2));

	}

	@Override
	public List<List<SHAOPState>> getGeneralTestData() {
		return Arrays.asList(Arrays.asList(state1, state1a),
				Arrays.asList(state2), Arrays.asList(state3),
				Arrays.asList(state4));
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList(
				"SHAOPState.*\\[\\].*ConnectionWithParties.*conn1.*conn2.*progresstime.*settings.*",
				"SHAOPState.*\\[act1\\].*ConnectionWithParties.*conn1.*conn2.*progresstime.*settings.*",
				"SHAOPState.*\\[\\].*ConnectionWithParties.*conn1.*conn2.*progressrounds.*settings.*",
				"SHAOPState.*\\[\\].*ConnectionWithParties.*conn1.*conn2.*progresstime.*settings.*");
	}

	@Test
	public void constructWith0Connection() {
		new SHAOPState(actions, new ProtocolToPartyConnections(Arrays.asList()),
				progresstime, settings, null, 0, null, null);
	}

	@Test
	public void constructWithNullConnection() {
		SHAOPState state = new SHAOPState(actions, null, progresstime, settings,
				null, 0, null, null);
		assertEquals(0, state.getConnections().size());
	}

	@Test
	public void testCurrentActor() {
		assertEquals(party1, state1.getCurrentTeam());
	}

	@Test
	public void testNextActor2() {
		ProtocolToPartyConnections conns = new ProtocolToPartyConnections(
				Arrays.asList(party2conn, party3conn));
		SHAOPState state = new SHAOPState(actions, conns, progresstime,
				settings, null, 0, partynrs, null);
		assertEquals(party2, state.getCurrentTeam());
	}

	@Test
	public void testNextActorAfterAction() {
		List<Action> actions = Arrays.asList(action);
		SHAOPState state = new SHAOPState(actions, connections, progresstime,
				settings, null, 0, null, null);
		assertEquals(party1, state.getCurrentTeam());
	}

	@Test
	public void isFinalTest() {
		assertFalse(state1.isFinal(NOW));
		SHAOPState state = new SHAOPState(actions, connections, progresstime,
				settings, mock(ProtocolException.class), 0, null, null);
		assertTrue(state.isFinal(NOW));
	}

	@Test
	public void withExceptionTest() {
		SHAOPState finalstate1 = state1.with(mock(ProtocolException.class));
		assertTrue(finalstate1.isFinal(NOW));
		SHAOPState finalstate2 = finalstate1
				.with(mock(ProtocolException.class));
		assertTrue(finalstate2.isFinal(NOW));
	}

	@Test
	public void isOfferAgreementTest1() {
		List<Action> actions = Arrays.asList(offer1);
		SHAOPState state = new SHAOPState(actions, connections3, progressrounds,
				settings, null, 0, null, null);
		assertTrue(state.getAgreements().getMap().isEmpty());
	}

	@Test
	public void isOfferAcceptAgreementTest() {
		List<Action> actions = Arrays.asList(offer1, accept2);
		SHAOPState state = new SHAOPState(actions, connections4, progressrounds,
				settings, null, 0, partynrs, null);
		assertFalse(state.getAgreements().getMap().isEmpty());
		assertTrue(state.isFinal(NOW));
	}

	@Test
	public void isOfferAcceptOfferAgreementTest() {
		List<Action> actions = Arrays.asList(offer1, accept1, offer1);
		SHAOPState state = new SHAOPState(actions, connections3, progressrounds,
				settings, null, 0, null, null);
		assert (state.getAgreements().getMap().isEmpty());
	}

	@Test
	public void isOfferAcceptAcceptotherAgreementTest() {
		List<Action> actions = Arrays.asList(offer1, accept1, acceptother);
		SHAOPState state = new SHAOPState(actions, connections3, progressrounds,
				settings, null, 0, null, null);
		assertTrue(state.getAgreements().getMap().isEmpty());
	}

	@Test(expected = IllegalArgumentException.class)
	public void withWrongActionTest() {
		// wrong because action has actor null which does not match party1
		SHAOPState state = state1.with(party1, action);
	}

	@Test(expected = IllegalArgumentException.class)
	public void withActionTest() {
		when(action.getActor()).thenReturn(party1);
		state1.with(party1, action);
	}

	@Test
	public void withConnectionTest() {
		SHAOPState state = state1.with(party3conn);
		assertEquals(3, state.getConnections().size());
	}

	@Test
	public void isEndNegotiationFinalTest() {
		List<Action> actions = Arrays.asList(endNegotiation1);
		SHAOPState state = new SHAOPState(actions, connections3, progressrounds,
				settings, null, 0, null, null);
		assertTrue(state.getAgreements().getMap().isEmpty());
		assertEquals(null, state.getError());
		assertTrue(state.isFinal(NOW));

	}

	@Test
	public void isProtocolErrorFinal() {
		SHAOPState state = new SHAOPState(actions, connections3, progressrounds,
				settings,
				new ProtocolException("test error", new PartyId("test")), 0,
				null, null);
		assertTrue(state.isFinal(NOW));
		assertNotNull(state.getError());
	}

	@Test(expected = IllegalArgumentException.class)
	public void RefuseImmediateAccept() {
		Accept nullaccept = new Accept(party1, null);
		SHAOPState state = state1.with(party1, nullaccept);
	}

	@Test(expected = IllegalArgumentException.class)
	public void RefuseNotMyTurn() {
		List<Action> actions = Arrays.asList(offer1);
		SHAOPState state = new SHAOPState(actions, connections3, progressrounds,
				settings, null, 1, partynrs, null);

		state.with(party1, offer1);
	}

	@Test(expected = IllegalArgumentException.class)
	public void RefuseNullAccept() {
		List<Action> actions = Arrays.asList(offer1);
		SHAOPState state = new SHAOPState(actions, connections3, progressrounds,
				settings, null, 0, partynrs, null);

		Accept nullaccept = new Accept(party2, null);
		state = state.with(party2, nullaccept);
		assertTrue(state.isFinal(NOW));
	}

	@Test(expected = IllegalArgumentException.class)
	public void RefuseNullAction() {
		List<Action> actions = Arrays.asList(offer1);
		SHAOPState state = new SHAOPState(actions, connections3, progressrounds,
				settings, null, 0, null, null);

		state = state.with(party2, null);
	}

	@Test
	public void testRoundNumberAfterActions() {

		SHAOPState state = new SHAOPState(actions, connections4,
				new ProgressRounds(10, 0, null), settings, null, 0, partynrs,
				null);

		assertEquals((Double) 0d,
				state.getProgress().get(System.currentTimeMillis()));
		Offer act1 = new Offer(party1, bid1);
		state = state.with(party1, act1);
		assertEquals((Double) 0d,
				state.getProgress().get(System.currentTimeMillis()));
		Offer act2 = new Offer(party3, bid1);
		state = state.with(party3, act2);
		assertEquals((Double) 0.1d,
				state.getProgress().get(System.currentTimeMillis()));

	}

	@Ignore // FIXME
	@Test
	public void getResultTest() {

		Map<PartyId, Double> spent = new HashMap<>();
		spent.put(party1, 0.1d);
		SHAOPState state = new SHAOPState(actions, connections4,
				new ProgressRounds(10, 0, null), settings, null, 0, partynrs,
				spent);
		SessionResult res = state.getResult();

	}

}
